#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#-------------------------------------------------------------------------------
"""Base setuptools command for tests."""
__author__ = ('Lance Finn Helsten',)
__version__ = '0.7'
__copyright__ = """Copyright (C) 2014 Lance Helsten"""
__docformat__ = "reStructuredText en"
__license__ = """
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import sys
if sys.version_info < (3, 0):
    raise Exception("test requires Python 3.0 or higher.")
import os
import io
import errno
import configparser
import shutil
import itertools
from importlib.abc import Finder, Loader
import re
import logging
import subprocess
import time
import unittest
from setuptools.command.test import test as SetupTest

__all__ = ['TestBase']

try:
    import coverage
except ImportError:
    print("### Ned Batchelder coverage module missing.", file=sys.stderr)
    print("### Tests will proceed with no coverage report.", file=sys.stderr)


class TestFinder(Finder, Loader):
    def __init__(self, testcommand):
        self.build_lib = testcommand.build_lib
        self.test_lib = testcommand.test_lib

    def find_module(self, fullname, path=None):
        print("-+-+-+-+", fullname, path)
        return None

    def load_module(self, fullname):
        return None


class TestBase(SetupTest):
    def initialize_options(self):
        self.suite = []
        self.debug = None
        self.bSetup = False
        self.bTeardown = False
        self.test_type = None
        self.suite_name = None
        self.coverage = False
        self.test_suite = None
        self.test_src = None
        self.build_base = 'build'
        self.build_lib = None
        self.test_base = None
        self.test_lib = None
        self.scripts_base = None

    def finalize_options(self):
        if self.test_type is None:
            raise ValueError("test_type must be set.")

        if self.suite_name is None:
            raise ValueError("suite_name must be set.")

        if self.test_src is None:
            self.test_src = 'test'

        if self.build_lib is None:
            self.build_lib = os.path.join(self.build_base, 'lib')

        if self.test_base is None:
            self.test_base = os.path.join(self.build_base, self.test_type)

        if self.test_lib is None:
            self.test_lib = os.path.join(self.test_base, 'lib')

        if self.scripts_base is None:
            self.scripts_base = os.path.join(self.build_base, "scripts-{0.version}".format(sys))

        #Reset the module load path
        del sys.path[0]
        sys.path.insert(0, os.path.dirname(os.path.dirname(__file__)))
        sys.path.insert(0, os.path.abspath(self.build_lib))
        sys.path.insert(0, os.path.abspath(self.test_lib))

        #Setup build/test directory
        if os.path.isdir(self.test_base):
            shutil.rmtree(self.test_base)
        os.makedirs(self.test_base, exist_ok=True)
        os.makedirs(os.path.join(self.test_base, 'etc'), exist_ok=True)

        for dirpath, dirnames, filenames in os.walk(self.test_src):
            for d in dirnames:
                dst = os.path.join(self.test_lib, dirpath, d)
                if not os.path.exists(dst):
                    #print('creating', dst)
                    os.makedirs(dst, exist_ok=True)
            for f in filenames:
                src = os.path.join(dirpath, f)
                dst = os.path.join(self.test_lib, dirpath, f)
                if not os.path.exists(dst) or os.path.getmtime(src) > os.path.getmtime(dst):
                    #print('copying', src, '->', dst)
                    shutil.copy2(src, dst)

        #Setup logging
        logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s',
            level=logging.NOTSET,
            filename=os.path.join(self.test_base, "logfile.log"))

        #Setup coverage
        if self.coverage:
            omit = []
            for dirpath, dirnames, filenames in os.walk(self.test_base):
                omit.append(os.path.join(dirpath, '*.py'))

            for dirpath, dirnames, filenames in os.walk(self.build_lib):
                omit.append(os.path.join(dirpath, '__init__.py'))

            if "VIRTUAL_ENV" in os.environ:
                for dirpath, dirnames, filenames in os.walk(os.environ["VIRTUAL_ENV"]):
                    omit.append(os.path.join(dirpath, '*.py'))

            datafile = os.path.join(self.build_base, "coverage_data", self.test_type)
            os.makedirs(os.path.dirname(datafile), exist_ok=True)
            self.cov = coverage.coverage(data_file=datafile, data_suffix=True, branch=True, config_file=True, omit=omit)
        else:
            self.cov = NoCoverage()

    def build_test_suite(self):
        #Clear the cache of anything to do with this build so it will reload.
        #If this is not done then strange ImportError are thrown.
        def list_tests(suite):
            ret = []
            for t in suite:
                if isinstance(t, unittest.TestSuite):
                    ret.extend(list_tests(t))
                else:
                    ret.append(t)
            return ret

        cacheclear = [i for i in sys.path_importer_cache
                        if os.path.abspath(i).startswith(os.getcwd())]
        for i in cacheclear:
            del sys.path_importer_cache[i]

        #Find all test suites to run
        if self.suite:
            self.suite = unittest.TestLoader().loadTestsFromName(self.suite)
        elif self.debug:
            self.suite = unittest.TestLoader().loadTestsFromName(self.debug)
            tests = list_tests(self.suite)
            if len(tests) < 1:
                raise DistutilsArgError("No test case to execute.")
            if len(tests) > 1:
                raise DistutilsArgError("Only one test case for debug allowed.")

            import importlib
            self.debugModule = importlib.import_module(tests[0].__class__.__module__)
            self.debugClass = getattr(self.debugModule, tests[0].__class__.__name__)
            self.debugSetUp = getattr(self.debugClass, "setUp")
            self.debugMethod = getattr(self.debugClass, tests[0]._testMethodName)
            self.debugTearDown = getattr(self.debugClass, "tearDown")
        else:
            self.cov.start()
            self.suite = getattr(__import__('test'), self.suite_name)()
            self.cov.stop()

    def run_test_suite(self):
        #Run the tests
        if self.debug is None:
            self.cov.start()
            tr = unittest.TextTestRunner()
            tr.run(self.suite)
            self.cov.stop()
            self.cov.save()
            #self.cov.report()
            try:
                self.cov.xml_report(outfile=os.path.join(self.build_base, self.test_type + '_coverage.xml'))
                self.cov.html_report(directory=os.path.join(self.build_base, self.test_type + '_htmlcov'))
            except coverage.misc.CoverageException as err:
                pass
        else:
            import pdb
            db = pdb.Pdb()
            db.rcLines.append('import {0.debugModule.__name__}'.format(self))
            if self.bSetup:
                db.rcLines.append('break {0.debugSetUp}'.format(self))
            db.rcLines.append('break {0.debugModule.__name__}.{0.debugClass.__name__}.{0.debugMethod.__name__}'.format(self))
            if self.bTeardown:
                db.rcLines.append('break {0.debugTearDown}'.format(self))
            db.rcLines.append('continue')
            db.rcLines.append('args')
            db.rcLines.append('list')
            db.runcall(self.suite.debug)

    def run(self):
        self.build_test_suite()
        self.run_test_suite()



class NoCoverage():
    """Class that duck types coverage but does not do anything."""
    def __init__(self, *argv, **kvarg): pass
    def analysis(self, morf): pass
    def analysis2(self, morf): pass
    def annotate(self, morfs=None, directory=None, ignore_errors=None, omit=None, include=None): pass
    def clear_exclude(which='exclude'): pass
    def combine(self): pass
    def erase(self): pass
    def exclude(self, regex, which='exclude'): pass
    def get_exclude_list(self, which='exclude'): pass
    def html_report(self, morfs=None, directory=None, ignore_errors=None, omit=None, include=None): pass
    def load(self): pass
    def report(self, morfs=None, show_missing=True, ignore_errors=None, file=None, omit=None, include=None): pass
    def save(self): pass
    def start(self): pass
    def stop(self): pass
    def sysinfo(self): pass
    def use_cache(self): pass
    def xml_report(morfs=None, outfile=None, ignore_errors=None, omit=None, include=None): pass





#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#-------------------------------------------------------------------------------
"""Setuptools command to execute all tests. This is useful for continuous
integration systems."""
__author__ = ('Lance Finn Helsten',)
__version__ = '0.7'
__copyright__ = """Copyright (C) 2014 Lance Helsten"""
__docformat__ = "reStructuredText en"
__license__ = """
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import sys
import setuptools

__all__ = ['TestRunner']


class TestRunner(setuptools.Command):
    description = "Run all unit and acceptance tests on the system."
    user_options = [
        ("suite=", "s", "Run specific test suite [default: all tests]."),
        ("level=", "l", "Test suite level to run: smoke, sanity, or shakedown."),
        ("debug=", "d", "Debug a specific test with preset breakpoints."),
        ("coverage", "c", "Turn on code coverage for the tests."),
        ("bSetup", None, "Add a breakpoint in setUp for debug."),
        ("bTeardown", None, "Add a breakpoint in tearDown for debug."),
    ]

    def initialize_options(self):
        self.suite = []
        self.level = "sanity"
        self.debug = None
        self.coverage = False
        self.bSetup = False
        self.bTeardown = False

    def finalize_options(self):
        if self.suite is None:
            raise ValueError("suite must be set.")

    def run(self):
        self.reinitialize_command('test_unit',
                                  suite=self.suite,
                                  debug=self.debug,
                                  coverage=self.coverage,
                                  bSetup=self.bSetup,
                                  bTeardown=self.bTeardown)
        self.run_command('test_unit')

        self.reinitialize_command('test_accept',
                                  suite=self.suite,
                                  level=self.level,
                                  debug=self.debug,
                                  coverage=self.coverage,
                                  bSetup=self.bSetup,
                                  bTeardown=self.bTeardown)
        self.run_command('test_accept')




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#-------------------------------------------------------------------------------
"""Setuptools command to execute acceptance tests."""
__author__ = ('Lance Finn Helsten',)
__version__ = '0.7'
__copyright__ = """Copyright (C) 2014 Lance Helsten"""
__docformat__ = "reStructuredText en"
__license__ = """
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from .testbase import TestBase

__all__ = ['AcceptTestRunner']


class AcceptTestRunner(TestBase):
    description = "Run all acceptance tests on the system."

    user_options = [
        ("suite=", "s", "Run specific test suite [default: all tests]."),
        ("level=", "l", "Test suite level to run: smoke, sanity, or shakedown."),
        ("debug=", "d", "Debug a specific test with preset breakpoints."),
        ("coverage", "c", "Turn on code coverage for the tests."),
        ("bSetup", None, "Add a breakpoint in setUp for debug."),
        ("bTeardown", None, "Add a breakpoint in tearDown for debug."),
    ]

    levels = {
        'smoke':'SmokeAcceptSuite',
        'sanity':'SanityAcceptSuite',
        'shakedown':'ShakedownAcceptSuite'
    }

    def initialize_options(self):
        super().initialize_options()
        self.test_type = "accept"
        self.level = "sanity"

    def finalize_options(self):
        if self.level not in self.levels.keys():
            raise DistutilsOptionError("Invalid test level: {0}".format(self.level))
        self.suite_name = self.levels[self.level]
        super().finalize_options()




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#-------------------------------------------------------------------------------
"""
StoryBox HTML5 writer for DocUtils.

The output conforms to the XHTML option of HTML 5. The CSS2 style sheet
must be available as `http://www.storybox.com/css/style-2010.css` for
proper viewing.
"""
__docformat__ = 'reStructuredText'

import sys
import os
import os.path
import re
from datetime import datetime, timedelta
import docutils
from docutils import frontend, nodes, utils, writers
from docutils.writers import html4css1

class XXWriter(html4css1.Writer):

	default_stylesheet = 'html5css2.css'

	default_stylesheet_path = utils.relative_path(
		os.path.join(os.getcwd(), 'dummy'),
		os.path.join(os.path.dirname(__file__), default_stylesheet))

	default_template = 'template.html'

	default_template_path = utils.relative_path(
		os.path.join(os.getcwd(), 'dummy'),
		os.path.join(os.path.dirname(__file__), default_template))

	settings_spec = html4css1.Writer.settings_spec + (
		'PEP/HTML-Specific Options',
		'For the PEP/HTML writer, the default value for the --stylesheet-path '
		'option is "%s", and the default value for --template is "%s". '
		'See HTML-Specific Options above.'
		% (default_stylesheet_path, default_template_path),
		(('Python\'s home URL.  Default is "http://python.org".',
		  ['--python-home'],
		  {'default': 'http://python.org', 'metavar': '<URL>'}),
		 ('Home URL prefix for PEPs.  Default is "." (current directory).',
		  ['--pep-home'],
		  {'default': '.', 'metavar': '<URL>'}),
		 # For testing.
		 (frontend.SUPPRESS_HELP,
		  ['--no-random'],
		  {'action': 'store_true', 'validator': frontend.validate_boolean}),))

	settings_default_overrides = {
			'input_encoding':'utf-8',
			'output_encoding':'UTF-8',
			'language_code':'en',
			'embed_stylesheet':False,
			'link-stylesheet':True,
			'stylesheet_path':None, #default_stylesheet_path,
			'template':default_template_path
		}

	relative_path_settings = (html4css1.Writer.relative_path_settings + ('template',))
	config_section = 'pep_html writer'
	config_section_dependencies = ('writers', 'html4css1 writer')

	def __init__(self, webroot="http://www.storybox.com", debug=False):
		super().__init__()
		self.webroot = webroot
		self.debug = debug
		self.translator_class = HTMLTranslator

	def apply_template(self):
		self.visitor.layout()
		return super().apply_template()

	def interpolation_dict(self):
		subs = super().interpolation_dict()
		subs["webroot"] = self.webroot
		subs["iso639"] = self.visitor.iso639
		if not self.debug:
			subs["debug_html"] = ""
			subs["debug_jpg"] = ""
			subs["debug_png"] = ""
			subs["debug_gif"] = ""
		else:
			subs["debug_html"] = ".html"
			subs["debug_jpg"] = ".jpg"
			subs["debug_png"] = ".png"
			subs["debug_gif"] = ".gif"
		return subs

	def assemble_parts(self):
		super().assemble_parts()


class HTMLTranslator(html4css1.HTMLTranslator):
	"""
	"""
	def __init__(self, document):
		super().__init__(document)
		self.iso639 = self.settings.language_code

		now = datetime.utcnow()
		now = now - timedelta(microseconds=now.microsecond)
		expire = now + timedelta(days=7)
		metadate = """<meta content="{0}Z" name="date" />""".format(now.isoformat())
		metaexp = """<meta content="{0}Z" http-equiv="expires" />""".format(expire.isoformat())
		self.meta.append(metadate)
		self.meta.append(metaexp)

	#def depart_field_list(self, node):
	#	html4css1.HTMLTranslator.depart_field_list(self, node)
	#	if 'rfc2822' in node['classes']:
	#		 self.body.append('<hr />\n')

	def layout(self, indent='    '):
		"""This will indent each new tag in the body by given number of spaces."""


		self.__indent(self.head, indent)
		self.__indent(self.meta, indent)
		self.__indent(self.stylesheet, indent)
		self.__indent(self.header, indent)
		self.__indent(self.body, indent, initial=3)
		self.__indent(self.footer, indent)
		self.__indent(self.body_pre_docinfo, indent, initial=3)
		self.__indent(self.docinfo, indent)

	def __indent(self, lines, indent, initial=2):
		"""This will indent the given set of lines by normal HTML layout.
		An initial indent of `2*indent` will be used to account for the
		`<html><head>` or `<html><body>` levels."""
		tagempty = re.compile(r"""<\w+(\s+[^>]*?)*/>""")
		tagopen = re.compile(r"""<\w+(\s+[^>]*?)*>""")
		tagclose = re.compile(r"""</\w+\s*>""")
		prefix = indent * initial
		for i in range(len(lines)):
			line = lines[i].strip()
			if tagempty.match(line):
				line = prefix + line
			elif tagopen.match(line):
				line = prefix + line
				prefix = prefix + indent
			elif tagclose.match(line):
				prefix = prefix[:-len(indent)]
				line = prefix + line
			elif '\n' in line:
				line = [prefix + l.strip() for l in line.split('\n')]
				line = os.linesep.join(line)
			else:
				line = prefix + line
			lines[i] = line + os.linesep





#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
#-------------------------------------------------------------------------------
"""Update version information in a project."""
__author__ = ('Lance Finn Helsten',)
__version__ = '0.7'
__copyright__ = """Copyright (C) 2014 Lance Helsten"""
__docformat__ = "reStructuredText en"
__license__ = """
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

__all__ = ['UpdateVersion']

import sys
if sys.version_info < (3, 3):
    raise Exception("upvers requires Python 3.3 or higher.")
import os
import shutil
import errno
import re
import setuptools
from distutils.version import StrictVersion


VERS_RE = re.compile(r"""^(?P<major>\d+)\.(?P<minor>\d+)(\.(?P<patch>\d+))?$""")


class UpdateVersion(setuptools.Command):
    description = "Modify project version by changing ``__version__`` tags."
    user_options = [
        ('set=', None, "Change version to given value."),
        ('major', None, "Move major number to next value set minor to 0."),
        ('minor', None, "Move minor number to next value remove patch."),
        ('patch', None, "Move patch number to next value."),
        ('alpha', None, "Make alpha or increment if already alpha."),
        ('beta', None, "Make beta or increment if already beta."),
        ('release', None, "Remove alpha or beta."),
    ]

    def initialize_options(self):
        self.set = None
        self.major = False
        self.minor = False
        self.patch = False
        self.alpha = False
        self.beta = False
        self.release = False

    def finalize_options(self):
        if self.set:
            try:
                self.set = StrictVersion(self.set)
            except ValueError as err:
                print("Error:", err, file=sys.stderr)
                sys.exit(errno.EINVAL)

            self.major = False
            self.minor = False
            self.patch = False
            self.alpha = False
            self.beta = False
            self.release = False
        else:
            self.major = bool(self.major)
            self.minor = bool(self.minor)
            self.patch = bool(self.patch)
            self.alpha = bool(self.alpha)
            self.beta = bool(self.beta)
            self.release = bool(self.release)

    def run(self):
        if self.set:
            newver = str(self.set)
        else:
            try:
                oldver = self.distribution.metadata.version
                oldver = StrictVersion(oldver)
            except ValueError as err:
                print("Error: setup.py", err, file=sys.stderr)
                sys.exit(errno.EINVAL)

            major, minor, patch = oldver.version
            pre = oldver.prerelease

            if self.alpha:
                if pre is None or pre[0] != 'a':
                    pre = ('a', 0)
                else:
                    pre = (pre[0], pre[1] + 1)
            elif self.beta:
                if pre is None or pre[0] != 'b':
                    pre = ('b', 0)
                else:
                    pre = (pre[0], pre[1] + 1)
            elif self.release:
                pre = None
            elif self.patch:
                patch = patch + 1
                pre = None
            elif self.minor:
                minor = minor + 1
                patch = 0
                pre = None
            elif self.major:
                major = major + 1
                minor = 0
                patch = 0
                pre = None
            else:
                return
            newver = StrictVersion()
            newver.version = (major, minor, patch)
            newver.prerelease = pre
            newver = str(newver)

        for dirpath, dirnames, filenames in os.walk(os.curdir):
            for filename in (f for f in filenames if os.path.splitext(f)[1] == '.py'):
                inpath = os.path.join(dirpath, filename)
                outpath = inpath + '.tmp'
                with open(inpath) as fin, open(outpath, 'w') as fout:
                    for line in fin:
                        if line.startswith("__version__"):
                            line = "__version__ = '{0}'\n".format(newver)
                        fout.write(line)
                shutil.copystat(inpath, outpath)
                os.replace(outpath, inpath)





#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
#-------------------------------------------------------------------------------
"""Implements a setuptools ``build_website`` command."""
__author__ = ('Lance Finn Helsten',)
__version__ = '0.7'
__copyright__ = """Copyright (C) 2014 Lance Helsten"""
__docformat__ = "reStructuredText en"
__license__ = """
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

__all__ = ['WebDistribution', 'BuildWebsite']

import sys
if sys.version_info < (3, 3):
    raise Exception("venv requires Python 3.3 or higher.")
import os
import errno
import shutil
import configparser
import subprocess
import setuptools

class WebDistribution(setuptools.Distribution):
    def __init__(self, attrs=None):
        self.websites = None
        super().__init__(attrs)

class BuildWebsite(setuptools.Command):
    """Generate the website from reStructured text files."""

    user_options = [
        ('force=', 'f', "Forcibly build everything (ignore file timestamps)"),
        ('debug', 'g', "Compile extensions and libraries with debugging information"),
        ('template=', None, "Path to template map file."),
        ('rst2html=', None, "Path to rst2html.py executable.")
        ]
    boolean_options = ['force', 'debug']

    def initialize_options(self):
        self.build_base = 'build'
        self.force = False
        self.debug = None
        self.template = None
        self.rst2html = None

    def finalize_options(self):
        self.set_undefined_options('build',
                ('force', 'force'),
                ('debug', 'debug'))

        if self.template == None:
            sys.exit("Path to template map file required.")
        self.template = os.path.expandvars(self.template)
        self.template = os.path.expanduser(self.template)
        self.template = os.path.abspath(self.template)
        with open(self.template) as f:
            self.template = eval(f.read())

        if self.rst2html is None:
            r2hout = subprocess.check_output(['python-config', '--prefix'])
            r2hout = str(r2hout, encoding='utf-8').strip()
            self.rst2html = os.path.join(r2hout, 'bin', 'rst2html.py')
        else:
            self.rst2html = os.path.expandvars(self.rst2html)
            self.rst2html = os.path.expanduser(self.rst2html)
            self.rst2html = os.path.abspath(self.rst2html)

    def run(self):
        for website in self.distribution.websites:
            self.copy_filetypes(website, '.html', 'HTML')
            self.copy_filetypes(website, '.xml', 'XML')
            self.copy_filetypes(website, '.xsd', 'XSD')
            self.copy_filetypes(website, '.css', 'CSS2')
            self.copy_filetypes(website, '.py', 'Python')
            self.copy_filetypes(website, '.js', 'JavaScript')
            self.copy_filetypes(website, '.jpg', 'JPEG')
            self.copy_filetypes(website, '.png', 'PNG')
            self.copy_filetypes(website, '.gif', 'GIF')
            self.copy_filetypes(website, '.ico', 'Favicon')
            self.copy_filetypes(website, '.yaml', 'YAML')
            self.generate_html(website)

    def generate_html(self, website):
        args = [self.rst2html,
                "--traceback",
                "TEMPLATE_ARG",
                "SRC_PATH_ARG_2",
                "DST_PATH_ARG_3"]

        configs = []
        for pathfrag, cfgpath in self.template.items():
            cfgmtime = os.path.getmtime(cfgpath)
            cfg = configparser.ConfigParser()
            cfg.read(cfgpath)
            tmppath = cfg["html4css1 writer"]["template"]
            tmpmtime = os.path.getmtime(tmppath)
            configs.append((pathfrag, cfgpath, max(cfgmtime, tmpmtime)))
        configs.sort(reverse=True)

        #Process reStructuredText files
        try:
            for dirpath, dirnames, filenames in os.walk(website):
                filenames = [os.path.join(dirpath, f) for f in filenames if os.path.splitext(f)[1] == '.rst']
                for f in filenames:
                    src = os.path.abspath(f)
                    srcmtime = os.path.getmtime(src)

                    dst = os.path.abspath(
                        os.path.join(self.build_base, os.path.splitext(f)[0] + ".html"))
                    dstmtime = os.path.getmtime(dst) if os.path.exists(dst) else 0

                    for pathfrag, cfg, cfgmtime in configs:
                        if dirpath.startswith(pathfrag):
                            break
                    else:
                        raise OSError("Unable to match configuration to path.")

                    args[-3] = "=".join(["--config", cfg])

                    if self.force or not os.path.isfile(dst) or srcmtime > dstmtime or cfgmtime > dstmtime:
                        if not os.path.exists(os.path.dirname(dst)):
                            os.makedirs(os.path.dirname(dst))
                        print("Generating HTML5/CSS2 for", f)
                        args[-2] = src
                        args[-1] = dst
                        ret = subprocess.call(args)
        except OSError as err:
            if err.errno == errno.ENOENT:
                print("error: Docutils missing.", file=sys.stderr)
            raise err

    def copy_filetypes(self, website, ext, mediatype):
        srcdir = os.path.join(self.build_base, website)
        for dirpath, dirnames, filenames in os.walk(website):
            files = [f for f in filenames if os.path.splitext(f)[1] == ext]
            for f in files:
                src = os.path.join(dirpath, f)
                dst = os.path.join(self.build_base, dirpath, f)
                if not os.path.isdir(os.path.dirname(dst)):
                    os.makedirs(os.path.dirname(dst))
                if (self.force
                or not os.path.isfile(dst)
                or os.path.getmtime(src) > os.path.getmtime(dst)):
                    print("Copy", mediatype, src)
                    shutil.copy(src, dst)

    def has_restructured_text(self):
        return True






#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
#-------------------------------------------------------------------------------
"""Setup and activate a python virtual environment through setup."""
__author__ = ('Lance Finn Helsten',)
__version__ = '0.7'
__copyright__ = """Copyright (C) 2014 Lance Helsten"""
__docformat__ = "reStructuredText en"
__license__ = """
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

__all__ = ['VirtualEnv']

import sys
if sys.version_info < (3, 3):
    raise Exception("venv requires Python 3.3 or higher.")
import os
import re
import venv
import ensurepip
import subprocess
import setuptools

class EnvBuilder(venv.EnvBuilder):
    def ensure_directories(self, env_dir):
        context = super().ensure_directories(env_dir)
        if os.path.islink(context.executable):
            real = os.path.realpath(context.executable)
            dirname, exename = os.path.split(os.path.abspath(real))
            context.executable = real
            context.python_dir = dirname
            context.python_exe = exename
            context.env_exe = os.path.join(context.bin_path, exename)
        return context


class VirtualEnv(setuptools.Command):
    description = "Setup the venv for this project."
    user_options = [
        ('venv-base=', None, "path to virtual environment directory (default: VENV)"),
        ('upgrade', 'U', "upgrade the packages"),
    ]

    def initialize_options(self):
        self.venv_base = None
        self.upgrade = False

    def finalize_options(self):
        if self.venv_base is None:
            self.venv_base = 'VENV'
        self.venv_base = os.path.abspath(self.venv_base)
        self.activate_path = os.path.join(self.venv_base, 'bin', 'activate')

    def run(self):

        ### Setup environment
        try:
            venvbuild = EnvBuilder(
                system_site_packages=False,
                clear=False,
                symlinks=True,
                upgrade=False,
                with_pip=True)
            venvbuild.create(self.venv_base)
        except Exception as err:
            print("Error: {0}".format(err), file=sys.stderr)
            return

        ### Load the virtual environment
        env_re = re.compile(r"""^(?P<key>\w+)=(?P<value>.*)$""")
        env_args = ['/bin/bash', '--posix']
        env_input = ['. {0.activate_path}'.format(self), 'set']
        env = subprocess.check_output(env_args, input=bytes('\n'.join(env_input), "utf8"))
        env = [env_re.match(x) for x in str(env, 'utf-8').split()]
        env = [(m.group('key'), m.group('value')) for m in env if m]
        env = dict(env)
        #env["PYTHONHOME"] = self.venv_base

        keys = list(env.keys())
        keys.sort()
        #for k in keys:
        #    print("{0}={1}".format(k, env[k]))

        ### Setup Required Packages
        def process_requires(name):
            value = getattr(self.distribution, name, [])
            if value is None:
                return
            if not isinstance(value, list):
                value = [value]
            for package in value:
                self.install_package(package, env)

        process_requires("setup_requires")
        process_requires("install_requires")
        process_requires("tests_require")

        ###
        print()
        print("Activate with: . {0}".format(os.path.relpath(self.activate_path)))


    def install_package(self, package, env):
        args = []
        args.append(os.path.join(self.venv_base, 'bin', 'pip'))
        args.append('install')
        if self.upgrade:
            args.append('--upgrade')
        args.append(package)
        subprocess.check_call(args, env=env, stdout=sys.stdout, stderr=sys.stderr)





